var Bcrypt = require('bcrypt'),
    Travelogue = require('travelogue'),
    LocalStrategy = require('passport-local').Strategy,
    FacebookStrategy = require('passport-facebook').Strategy,
    TwitterStrategy = require('passport-twitter').Strategy;

module.exports = function () {
  this.server.pack._settings.requirePath = __dirname + '/../../node_modules';
  this.server.pack.require(
    this.settings.plugins,
    function (err) { if (err) { throw err; } }
  );
  this.server.auth.strategy('passport', 'passport');
  var self = this;
  var Passport = this.server.plugins.travelogue.passport;

  Passport.use(
    new LocalStrategy(
      this.settings.plugins.travelogue.local,
      function (username, password, done) {
        // Check here classic login from your database/API/...
        if (self.users[username]) {
          Bcrypt.compare(password, self.users[username].password, function (err, isValid) {
            return done(null, isValid ? self.users[username] : false);
          });
        } else {
          return done(null, false);
        }
      }
    )
  );
  var mySocialAuthHandler = function (user, done) {
    // Check here social login from your database/API/...

    if (self.users[user.username]) {
      if (!self.users[user.username].social[user.provider]) {
        self.users[user.username].social[user.provider] = user.id;
      }
      return done(null, self.users[user.username]);

    } else {
      self.users[user.username] = { username: user.username };
      self.users[user.username].social = {};
      self.users[user.username].social[user.provider] = user.id;
      return done(null, self.users[user.username]);
    }
  };
  Passport.use(
    new FacebookStrategy(
      this.settings.plugins.travelogue.facebook,
      function (accessToken, refreshToken, profile, done) {
        profile.provider = 'facebook';
        mySocialAuthHandler(profile, done);
      }
    )
  );
  Passport.use(new TwitterStrategy(
    this.settings.plugins.travelogue.twitter,
    function(token, tokenSecret, profile, done) {
      profile.provider = 'twitter';
      mySocialAuthHandler(profile, done);
    }
  ));
  Passport.serializeUser(function(user, done) {
    done(null, user);
  });
  Passport.deserializeUser(function(obj, done) {
    done(null, obj);
  });

};